'''
Library to handle the manipulation and generation of CWL tool
'''

#  Import  ------------------------------

# General libraries
import os
import argparse
import sys
import logging

# External libraries
import ruamel.yaml
import six
from .version import __version__

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)

#  Constant(s)  ------------------------------

CWL_SHEBANG = "#!/usr/bin/env cwl-runner"
CWL_VERSIONS = ['draft-2', 'draft-3.dev1', 'draft-3.dev2', 'draft-3.dev3',
                'draft-3.dev4', 'draft-3.dev5', 'draft-3', 'draft-4.dev1',
                'draft-4.dev2', 'draft-4.dev3', 'v1.0.dev4', 'v1.0', None]
CWL_TYPE = ['null', 'boolean', 'int', 'long', 'float', 'double', 'string', 'File',
            'Directory', None]

#  Function(s)  ------------------------------

#  Class(es)  ------------------------------


class CommandLineTool(object):
    '''
    Contain all informations to describe a CWL command line tool.
    '''

    def __init__(self, tool_id=None, base_command=None, label=None, doc=None,
                 cwl_version=None, stdin=None, stderr=None, stdout=None):
        '''
        :param tool_id: unique identifier for this tool
        :type tool_id: STRING
        :param base_command: command line for the tool
        :type base_command: STRING
        :param label: label of this tool
        :type label: STRING
        :param doc: documentation for the tool, usually longer than the label
        :type doc: STRING
        :param cwl_version: version of the CWL tool
        :type cwl_version: STRING
        :param stdin: path to a file whose contents must be piped into stdin
        :type stdin: STRING
        :param stderr: capture stderr into the given file
        :type stderr: STRING
        :param stdout: capture stdout into the given file
        :type stdout: STRING

        inputs (:class:`cwlgen.CommandInputParameter` objects),
        outputs (:class:`cwlgen.CommandOutputParameter` objects),
        arguments (:class:`cwlgen.CommandLineBinding` objects),
        and requirements (:class:`cwlgen.Requirement` objects)
        are stored in lists which are initialized empty.
        '''
        self.id = tool_id
        self.baseCommand = base_command
        self.label = label
        self.doc = doc
        if cwl_version not in CWL_VERSIONS:
            LOGGER.warning("CWL version is not recognized as a valid version.")
            cwl_version = None
        self.cwlVersion = cwl_version
        self.stdin = stdin
        self.stderr = stderr
        self.stdout = stdout
        self.inputs = []  # List of [CommandInputParameter] objects
        self.outputs = []  # List of [CommandOutputParameter] objects
        self.arguments = []  # List of [CommandLineBinding] objects
        self.requirements = []  # List of Several object inhereting from [Requirement]
        self.hints = []
        self.successCodes = []
        self.temporaryFailCodes = []
        self.permanentFailCodes = []

    def export(self, outfile=None):
        '''
        Export the tool in CWL either on STDOUT or in outfile.
        '''
        cwl_tool = {k: v for k, v in vars(self).items() if v is not None and v != []}
        cwl_tool['class'] = 'CommandLineTool'
        # Add Inputs
        if self.inputs:
            cwl_tool['inputs'] = {}
            for in_param in self.inputs:
                cwl_tool['inputs'][in_param.id] = in_param.get_dict()

        # Add Outputs
        if self.outputs:
            cwl_tool['outputs'] = {}
            for out_param in self.outputs:
                cwl_tool['outputs'][out_param.id] = out_param.get_dict()

        # Write CWL file in YAML
        if outfile is None:
            six.print_(CWL_SHEBANG, "\n", sep='')
            six.print_(ruamel.yaml.dump(cwl_tool, Dumper=ruamel.yaml.RoundTripDumper))
        else:
            out_write = open(outfile, 'w')
            out_write.write(CWL_SHEBANG + '\n\n')
            out_write.write(ruamel.yaml.dump(cwl_tool, Dumper=ruamel.yaml.RoundTripDumper))
            out_write.close()


class Parameter(object):
    '''
    A parameter (common field of Input and Output) for a CommandLineTool
    '''

    def __init__(self, param_id, label=None, secondary_files=None, param_format=None,
                 streamable=False, doc=None, param_type=None):
        '''
        :param param_id: unique identifier for this parameter
        :type param_id: STRING
        :param label: short, human-readable label
        :type label: STRING
        :param secondary_files: If type is a file, describes files that must be
                                included alongside the primary file(s)
        :type secondary_files: STRING
        :param param_format: If type is a file, uri to ontology of the format or exact format
        :type param_format: STRING
        :param streamable: If type is a file, true indicates that the file is read or written
                           sequentially without seeking
        :type streamable: BOOLEAN
        :param doc: documentation
        :type doc: STRING
        :param param_type: type of data assigned to the parameter
        :type param_type: STRING corresponding to CWLType
        '''
        if param_type not in CWL_TYPE:
            LOGGER.warning("The type is incorrect for the parameter.")
            param_type = None
        self.id = param_id
        self.label = label
        self.secondaryFiles = secondary_files
        self.format = param_format
        self.streamable = streamable
        self.doc = doc
        self.type = param_type

    def get_dict(self):
        '''
        Transform the object to a [DICT] to write CWL.

        :return: dictionnary of the object
        :rtype: DICT
        '''
        dict_param = {k: v for k, v in vars(self).items() if v is not None and v is not False}
        if dict_param['type'] != 'File':
            # Remove what is only for File
            for key in ['format', 'secondaryFiles', 'streamable']:
               try:
                   del(dict_param[key])
               except KeyError:
                   pass
        return dict_param


class CommandInputParameter(Parameter):
    '''
    An input parameter for a :class:`cwlgen.CommandLineTool`.
    '''

    def __init__(self, param_id, label=None, secondary_files=None, param_format=None,
                 streamable=False, doc=None, input_binding=None, default=None, param_type=None):
        '''
        :param param_id: unique identifier for this parameter
        :type param_id: STRING
        :param label: short, human-readable label
        :type label: STRING
        :param secondary_files: If type is a file, describes files that must be
                                included alongside the primary file(s)
        :type secondary_files: STRING
        :param param_format: If type is a file, uri to ontology of the format or exact format.
        :type param_format: STRING
        :param streamable: If type is a file, true indicates that the file is read or written
                           sequentially without seeking
        :type streamable: BOOLEAN
        :param doc: documentation
        :type doc: STRING
        :param input_binding: describes how to handle the input
        :type input_binding: :class:`cwlgen.CommandLineBinding` object
        :param default: default value
        :type default: STRING
        :param param_type: type of data assigned to the parameter corresponding to CWLType
        :type param_type: STRING
        '''
        Parameter.__init__(self, param_id=param_id, label=label,
                           secondary_files=secondary_files, param_format=param_format,
                           streamable=streamable, doc=doc, param_type=param_type)
        self.inputBinding = input_binding
        self.default = default

    def get_dict(self):
        '''
        Transform the object to a [DICT] to write CWL.

        :return: dictionnary of the object
        :rtype: DICT
        '''
        dict_in = Parameter.get_dict(self)
        if self.inputBinding:
            dict_in['inputBinding'] = self.inputBinding.get_dict()
        return dict_in


class CommandOutputParameter(Parameter):
    '''
    An output parameter for a :class:`cwlgen.CommandLineTool`.
    '''

    def __init__(self, param_id, label=None, secondary_files=None, param_format=None,
                 streamable=False, doc=None, output_binding=None, param_type=None):
        '''
        :param param_id: unique identifier for this parameter
        :type param_id: STRING
        :param label: short, human-readable label
        :type label: STRING
        :param secondary_files: If type is a file, describes files that must be
                                included alongside the primary file(s)
        :type secondary_files: STRING
        :param param_format: If type is a file, uri to ontology of the format or exact format
        :type param_format: STRING
        :param streamable: If type is a file, true indicates that the file is read or written
                           sequentially without seeking
        :type streamable: BOOLEAN
        :param doc: documentation
        :type doc: STRING
        :param output_binding: describes how to handle the output
        :type output_binding: :class:`cwlgen.CommandOutputBinding` object
        :param param_type: type of data assigned to the parameter corresponding to CWLType
        :type param_type: STRING
        '''
        Parameter.__init__(self, param_id, label, secondary_files, param_format, streamable,
                           doc, param_type)
        self.outputBinding = output_binding

    def get_dict(self):
        '''
        Transform the object to a [DICT] to write CWL.

        :return: dictionnary of the object
        :rtype: DICT
        '''
        dict_out = Parameter.get_dict(self)
        if self.outputBinding:
            dict_out['outputBinding'] = self.outputBinding.get_dict()
        return dict_out


class CommandLineBinding(object):
    '''
    Describes how the handle an input or an argument.
    '''

    def __init__(self, load_contents=False, position=None, prefix=None, separate=False,
                 item_separator=None, value_from=None, shell_quote=False):
        '''
        :param load_contents: Read up to the fist 64 KiB of text from the file and
                              place it in the "contents" field of the file object
        :type load_contents: BOOLEAN
        :param position: The sorting key
        :type positio: INT
        :param prefix: Command line prefix to add before the value
        :type prefix: STRING
        :param separate:
        :type separate: BOOLEAN
        :param item_separator: Join the array elements into a single string separated by this item
        :type item_separator: STRING
        :param value_from: Use this as the value
        :type value_from: STRING
        :param shell_quote: Value is quoted on the command line
        :type shell_quote: BOOLEAN
        '''
        self.loadContents = load_contents
        self.position = position
        self.prefix = prefix
        self.separate = separate
        self.itemSeparator = item_separator
        self.valueFrom = value_from
        self.shellQuote = shell_quote

    def get_dict(self):
        '''
        Transform the object to a [DICT] to write CWL.

        :return: dictionnary of the object
        :rtype: DICT
        '''
        dict_binding = {k: v for k, v in vars(self).items() if v is not None and v is not False}
        return dict_binding


class CommandOutputBinding(object):
    '''
    Describes how to generate an output parameter based on the files produced.
    '''

    def __init__(self, glob=False, load_contents=False, output_eval=None):
        '''
        :param glob: Find corresponding file(s)
        :type glob: STRING
        :param load_contents: For each file matched, read up to the 1st 64 KiB of text and
                              place it in the contents field
        :type load_contents: BOOLEAN
        :param output_eval: Evaluate an expression to generate the output value
        :type output_eval: STRING
        '''
        self.glob = glob
        self.loadContents = load_contents
        self.outputEval = output_eval

    def get_dict(self):
        '''
        Transform the object to a [DICT] to write CWL.

        :return: dictionnary of the object
        :rtype: DICT
        '''
        dict_binding = {k: v for k, v in vars(self).items() if v is not None and v is not False}
        return dict_binding


class Requirement(object):
    '''
    Requirement that must be met in order to execute the process.
    '''

    def __init__(self, req_class):
        '''
        :param req_class: requirement class
        :type req_class: STRING
        '''
        self.req_class = req_class


class InlineJavascriptReq(Requirement):
    '''
    Workflow platform must support inline Javascript expressions.
    '''

    def __init__(self, expression_lib=None):
        '''
        :param expression_lib: List of Strings
        :type expression_lib: STRING
        '''
        Requirement.__init__(self, 'InlineJavascriptRequirement')
        self.expressionLib = expression_lib


class DockerRequirement(Requirement):
    '''
    Workflow component should be run in a Docker container.
    This class specifies how to fetch or build the image.
    '''

    def __init__(self, docker_pull=None, docker_load=None, docker_file=None,
                 docker_import=None, docker_image_id=None, docker_output_dir=None):
        '''
        :param docker_pull: image to retrive with docker pull
        :type docker_pull: STRING
        :param docker_load: HTTP URL from which to download Docker image
        :type docker_load: STRING
        :param docker_file: supply the contents of a Dockerfile
        :type docker_file: STRING
        :param docker_import: HTTP URL to download and gunzip a Docker images
        :type docker_import: STRING
        :param docker_image_id: Image id for docker run
        :type docker_image_id: STRING
        :param docker_output_dir: designated output dir inside the Docker container
        :type docker_output_dir: STRING
        '''
        Requirement.__init__(self, 'DockerRequirement')
        self.dockerPull = docker_pull
        self.dockerLoad = docker_load
        self.dockerFile = docker_file
        self.dockerImport = docker_import
        self.dockerImageId = docker_image_id
        self.dockerOutputDir = docker_output_dir



'''
Library to handle the manipulation and generation of CWL tool
'''

#  Import  ------------------------------

# General libraries
import os
import argparse
import sys
import logging

# External libraries
import ruamel.yaml as ryaml
import six
import cwlgen

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

#  Class(es)  ------------------------------


class CWLToolParser(object):
    """
    Class to import content from an existing CWL Tool.
    """

    def _init_tool(self, cwl_dict):
        """
        Init tool from existing CWL tool.

        :param cwl_dict: Full content of CWL file
        :type cwl_dict: DICT
        """
        return cwlgen.CommandLineTool()

    def _load_id(self, tool, id_el):
        """
        Load the content of id into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param id_el: Content of id
        :type id_el: STRING or [STRING]
        """
        tool.id = id_el

    def _load_baseCommand(self, tool, command_el):
        """
        Load the content of baseCommand into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param command_el: Content of baseCommand
        :type command_el: STRING
        """
        tool.baseCommand = command_el

    def _load_label(self, tool, label_el):
        """
        Load the content of label into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param label_el: Content of label
        :type label_el: STRING
        """
        tool.label = label_el

    def _load_doc(self, tool, doc_el):
        """
        Load the content of doc into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param doc_el: Content of doc
        :type doc_el: STRING
        """
        tool.doc = doc_el

    def _load_cwlVersion(self, tool, cwl_version_el):
        """
        Load the content of cwlVersion into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param cwl_version_el: Content of cwlVersion
        :type cwl_version_el: STRING
        """
        tool.cwlVersion = cwl_version_el

    def _load_stdin(self, tool, stdin_el):
        """
        Load the content of stdin into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param stdin_el: Content of stdin
        :type stdin_el: STRING
        """
        tool.stdin = stdin_el

    def _load_stderr(self, tool, stderr_el):
        """
        Load the content of stderr into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param stderr_el: Content of stderr
        :type stderr_el: STRING
        """
        tool.stderr = stderr_el

    def _load_stdout(self, tool, stdout_el):
        """
        Load the content of stdout into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param stdout_el: Content of stdout
        :type stdout_el: STRING
        """
        tool.stdout = stdout_el

    def _load_class(self, tool, class_el):
        """
        Display message to inform that cwlgen only deal with CommandLineTool for the moment.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param class_el: Content of class
        :type class_el: STRING
        """
        if class_el != 'CommandLineTool':
            logger.warning('cwlgen library only handle CommandLineTool for the moment')

    def _load_inputs(self, tool, inputs_el):
        """
        Load the content of inputs into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param inputs_el: Content of inputs
        :type inputs_el: LIST or DICT
        """
        inp_parser = InputsParser()
        inp_parser.load_inputs(tool.inputs, inputs_el)

    def _load_outputs(self, tool, outputs_el):
        """
        Load the content of inputs into the tool.

        :param tool: Tool object from cwlgen
        :type tool: :class:`cwlgen.CommandLineTool`
        :param outputs_el: Content of outputs
        :type outputs_el: LIST or DICT
        """
        inp_parser = OutputsParser()
        inp_parser.load_outputs(tool.outputs, outputs_el)

    def import_cwl(self, cwl_path):
        """
        Load content of cwl into the :class:`cwlgen.CommandLineTool` object.

        :param cwl_path: Path of the CWL tool to be loaded.
        :type cwl_path: STRING
        :return: CWL tool content in cwlgen model.
        :rtype: :class:`cwlgen.CommandLineTool`
        """
        with open(cwl_path) as yaml_file:
            cwl_dict = ryaml.load(yaml_file, Loader=ryaml.Loader)
        tool = self._init_tool(cwl_dict)
        for key, element in cwl_dict.items():
            try:
                getattr(self, '_load_{}'.format(key))(tool, element)
            except AttributeError:
                logger.warning(key + " content is not processed (yet).")
        return tool


class InputsParser(object):
    """
    Class to parse content of inputs from an existing CWL Tool.
    """

    def _load_label(self, input_obj, label_el):
        """
        Load the content of type into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param label_el: Content of label
        :type label_el: STRING
        """
        input_obj.label = label_el

    def _load_secondaryFiles(self, input_obj, secfile_el):
        """
        Load the content of secondaryFiles into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param secfile_el: Content of secondaryFile
        :type secfile_el: STRING
        """
        input_obj.secondaryFile = secfile_el

    def _load_format(self, input_obj, format_el):
        """
        Load the content of format into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param format_el: Content of format
        :type format_el: STRING
        """
        input_obj.format = format_el

    def _load_streamable(self, input_obj, stream_el):
        """
        Load the content of streamable into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param stream_el: Content of streamable
        :type type_el: BOOLEAN
        """
        input_obj.streamable = stream_el

    def _load_doc(self, input_obj, doc_el):
        """
        Load the content of doc into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param doc_el: Content of doc
        :type doc_el: STRING
        """
        input_obj.doc = doc_el

    def _load_inputBinding(self, input_obj, inpb_el):
        """
        Load the content of inputBinding into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param inpb_el: Content of inputBinding
        :type inpb_el: DICT
        """
        ibparser = InputBindingParser()
        ibparser.load_inbinding(input_obj, inpb_el)

    def _load_default(self, input_obj, def_el):
        """
        Load the content of default into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param default_el: Content of default
        :type default_el: STRING
        """
        input_obj.default = def_el

    def _load_type(self, input_obj, type_el):
        """
        Load the content of type into the input object.

        :param input_obj: input obj
        :type input_obj: : :class:`cwlgen.CommandInputParameter`
        :param type_el: Content of type
        :type type_el: STRING
        """
        input_obj.type = type_el

    def load_inputs(self, inputs, inputs_el):
        """
        Load the content of inputs into the inputs list.

        :param inputs: list to store inputs
        :type inputs: LIST
        :param inputs_el: Content of inputs
        :type inputs_el: LIST or DICT
        """
        # For the moment, only deal with the format exported by cwlgen
        for key, value in inputs_el.items():
            input_obj = cwlgen.CommandInputParameter(key)
            for key, element in value.items():
                try:
                    getattr(self, '_load_{}'.format(key))(input_obj, element)
                except AttributeError:
                    logger.warning(key + " content for input is not processed (yet).")
            inputs.append(input_obj)


class InputBindingParser(object):
    """
    Class to parse content of inputBinding of input from existing CWL Tool.
    """

    def _load_loadContents(self, inbinding_obj, loadcontents_el):
        """
        Load the content of loadContents into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param loadcontents_el: Content of loadContents
        :loadContents loadcontents_el: BOOLEAN
        """
        inbinding_obj.loadContents = loadcontents_el

    def _load_position(self, inbinding_obj, position_el):
        """
        Load the content of position into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param position_el: Content of loadContents
        :loadContents position_el: INT
        """
        inbinding_obj.position = position_el

    def _load_prefix(self, inbinding_obj, prefix_el):
        """
        Load the content of prefix into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param prefix_el: Content of loadContents
        :loadContents prefix_el: STRING
        """
        inbinding_obj.prefix = prefix_el

    def _load_separate(self, inbinding_obj, separate_el):
        """
        Load the content of separate into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param separate_el: Content of loadContents
        :loadContents separate_el: BOOLEAN
        """
        inbinding_obj.separate = separate_el

    def _load_itemSeparator(self, inbinding_obj, itemsep_el):
        """
        Load the content of itemSeparator into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param itemsep_el: Content of loadContents
        :loadContents itemsep_el: STRING
        """
        inbinding_obj.itemSeparator = itemsep_el

    def _load_valueFrom(self, inbinding_obj, valuefrom_el):
        """
        Load the content of valueFrom into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param valuefrom_el: Content of loadContents
        :loadContents valuefrom_el: STRING
        """
        inbinding_obj.valueFrom = valuefrom_el

    def _load_shellQuote(self, inbinding_obj, shellquote_el):
        """
        Load the content of shellQuote into the inputBinding object.

        :param inbinding_obj: input_binding object
        :loadContents inbinding_obj: : :class:`cwlgen.CommandLineBinding`
        :param shellquote_el: Content of loadContents
        :loadContents shellquote_el: BOOLEAN
        """
        inbinding_obj.shellQuote = shellquote_el

    def load_inbinding(self, input_obj, inbinding_el):
        """
        Load the content of inputBinding into the input object.

        :param input_obj: input object
        :type input_obj: :class:`cwlgen.CommandInputParameter`
        :param inbinding_el: Content of inputs
        :type inbinding_el: DICT
        """
        inbinding_obj = cwlgen.CommandLineBinding()
        for key, value in inbinding_el.items():
            try:
                getattr(self, '_load_{}'.format(key))(inbinding_obj, value)
            except AttributeError:
                logger.warning(key + " content for inputBinding is not processed (yet).")
            input_obj.inputBinding = inbinding_obj


class OutputsParser(object):
    """
    Class to parse content of outputs from an existing CWL Tool.
    """

    def _load_label(self, output_obj, label_el):
        """
        Load the content of type into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandInputParameter`
        :param label_el: Content of label
        :type label_el: STRING
        """
        output_obj.label = label_el

    def _load_secondaryFiles(self, output_obj, secfile_el):
        """
        Load the content of secondaryFiles into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandInputParameter`
        :param secfile_el: Content of secondaryFile
        :type secfile_el: STRING
        """
        output_obj.secondaryFile = secfile_el

    def _load_format(self, output_obj, format_el):
        """
        Load the content of format into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandInputParameter`
        :param format_el: Content of format
        :type format_el: STRING
        """
        output_obj.format = format_el

    def _load_streamable(self, output_obj, stream_el):
        """
        Load the content of streamable into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandInputParameter`
        :param stream_el: Content of streamable
        :type type_el: BOOLEAN
        """
        output_obj.streamable = stream_el

    def _load_doc(self, output_obj, doc_el):
        """
        Load the content of doc into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandInputParameter`
        :param doc_el: Content of doc
        :type doc_el: STRING
        """
        output_obj.doc = doc_el

    def _load_outputBinding(self, output_obj, outpb_el):
        """
        Load the content of outputBinding into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandOutputParameter`
        :param outpb_el: Content of outputBinding
        :type outpb_el: DICT
        """
        obparser = OutputBindingParser()
        obparser.load_outbinding(output_obj, outpb_el)

    def _load_type(self, output_obj, type_el):
        """
        Load the content of type into the output object.

        :param output_obj: output obj
        :type output_obj: : :class:`cwlgen.CommandInputParameter`
        :param type_el: Content of type
        :type type_el: STRING
        """
        output_obj.type = type_el

    def load_outputs(self, outputs, outputs_el):
        """
        Load the content of outputs into the outputs list.

        :param outputs: list to store outputs
        :type outputs: LIST
        :param outputs_el: Content of outputs
        :type outputs_el: LIST or DICT
        """
        # For the moment, only deal with the format exported by cwlgen
        for key, value in outputs_el.items():
            output_obj = cwlgen.CommandOutputParameter(key)
            for key, element in value.items():
                try:
                    getattr(self, '_load_{}'.format(key))(output_obj, element)
                except AttributeError:
                    logger.warning(key + " content for output is not processed (yet).")
            outputs.append(output_obj)


class OutputBindingParser(object):
    """
    Class to parse content of outputBinding of output from existing CWL Tool.
    """

    def _load_glob(self, outbinding_obj, glob_el):
        """
        Load the content of glob into the outputBinding object.

        :param outbinding_obj: outputBinding object
        :glob outbinding_obj: : :class:`cwlgen.CommandOutputBinding`
        :param glob_el: Content of glob
        :glob glob_el: STRING
        """
        outbinding_obj.glob = glob_el

    def _load_loadContents(self, outbinding_obj, loadcontents_el):
        """
        Load the content of loadContents into the outputBinding object.

        :param outbinding_obj: outputBinding object
        :loadContents outbinding_obj: : :class:`cwlgen.CommandOutputBinding`
        :param loadcontents_el: Content of loadContents
        :loadContents loadcontents_el: BOOLEAN
        """
        outbinding_obj.loadContents = loadcontents_el

    def _load_outputEval(self, outbinding_obj, outputeval_el):
        """
        Load the content of outputEval into the outputBinding object.

        :param outbinding_obj: outputBinding object
        :outputEval outbinding_obj: : :class:`cwlgen.CommandOutputBinding`
        :param outputeval_el: Content of outputEval
        :outputEval outputeval_el: STRING
        """
        outbinding_obj.outputEval = outputeval_el

    def load_outbinding(self, output_obj, outbinding_el):
        """
        Load the content of outputBinding into the output object.

        :param output_obj: output object
        :type output_obj: :class:`cwlgen.CommandOutputParameter`
        :param outbinding_el: Content of outputBinding element
        :type outbinding_el: DICT
        """
        outbinding_obj = cwlgen.CommandOutputBinding()
        for key, value in outbinding_el.items():
            try:
                getattr(self, '_load_{}'.format(key))(outbinding_obj, value)
            except AttributeError:
                logger.warning(key + " content for outputBinding is not processed (yet).")
            output_obj.outputBinding = outbinding_obj



#!/usr/bin/env python

'''
Unit tests for cwlgen library
'''

#  Import  ------------------------------ 

# General libraries
import os
import filecmp
import unittest

# External libraries
import cwlgen

#  Class(es)  ------------------------------ 

class TestCommandLineTool(unittest.TestCase):

    def setUp(self):
        self.cwl = cwlgen.CommandLineTool(tool_id='an_id', label='a description '+\
                                         'with spaces.', base_command='a_command')

    def test_init(self):
        self.assertEqual(self.cwl.id, 'an_id')
        self.assertEqual(self.cwl.label, 'a description with spaces.')
        self.assertEqual(self.cwl.baseCommand, 'a_command')
        self.assertListEqual(self.cwl.inputs, [])
        self.assertListEqual(self.cwl.outputs, [])
        self.assertIsNone(self.cwl.doc)

    """
    def test_export(self):
        tmp_file = 'test_export.tmp'
        expected_file = os.path.dirname(__file__) + '/test_export.cwl'
        self.cwl.export(tmp_file)
        try:
            self.assertTrue(filecmp.cmp(expected_file, tmp_file))
        finally:
            os.remove(tmp_file)

    def test_full_export(self):
        tmp_file = 'test_full_export.tmp'
        expected_file = os.path.dirname(__file__) + '/test_full_export.cwl'
        self.cwl.doc = "documentation"
        an_input = cwlgen.CommandInputParameter('an_in_id', param_type='File')
        self.cwl.inputs.append(an_input)
        an_output = cwlgen.CommandOutputParameter('an_out_id', param_type='File')
        self.cwl.outputs.append(an_output)
        self.cwl.export(tmp_file)
        try:
            self.assertTrue(filecmp.cmp(expected_file, tmp_file))
        finally:
            os.remove(tmp_file)
    """

class TestParameter(unittest.TestCase):

    def setUp(self):
        self.param = cwlgen.Parameter('an_id', param_type='File', label='a_label',\
                                     doc='a_doc', param_format='a_format',\
                                     streamable=True, secondary_files='sec_files')

    def test_init(self):
        self.assertEqual(self.param.id, 'an_id')
        self.assertEqual(self.param.type, 'File')
        self.assertEqual(self.param.doc, 'a_doc')
        self.assertEqual(self.param.format, 'a_format')
        self.assertEqual(self.param.label, 'a_label')
        self.assertEqual(self.param.secondaryFiles, 'sec_files')
        self.assertTrue(self.param.streamable)

    def test_get_dict(self):
        dict_test = self.param.get_dict()
        self.assertEqual(dict_test['type'], 'File')
        self.assertEqual(dict_test['doc'], 'a_doc')
        self.assertEqual(dict_test['format'], 'a_format')
        self.assertEqual(dict_test['label'], 'a_label')
        self.assertEqual(dict_test['secondaryFiles'], 'sec_files')
        self.assertTrue(dict_test['streamable'])


class TestCommandInputParameter(unittest.TestCase):

    def setUp(self):
        self.frst_inp = cwlgen.CommandInputParameter('frst_id', param_type='File',\
                                                    label='a_label', \
                                                    default='def_value')
        binding = cwlgen.CommandLineBinding(position=2, prefix='--prefix')
        self.scnd_inp = cwlgen.CommandInputParameter('scnd_id', param_type='File',\
                                                    input_binding=binding)

    def test_init(self):
        # Test first input
        self.assertEqual(self.frst_inp.id, 'frst_id')
        self.assertEqual(self.frst_inp.type, 'File')
        self.assertEqual(self.frst_inp.label, 'a_label')
        self.assertEqual(self.frst_inp.default, 'def_value')
        # Test second input
        self.assertEqual(self.scnd_inp.id, 'scnd_id')
        self.assertEqual(self.scnd_inp.type, 'File')
        self.assertEqual(self.scnd_inp.inputBinding.position, 2)
        self.assertEqual(self.scnd_inp.inputBinding.prefix, '--prefix')

    def test_get_dict(self):
        # Test first input
        dict_frst = self.frst_inp.get_dict()
        self.assertEqual(dict_frst['type'], 'File')
        self.assertEqual(dict_frst['default'], 'def_value')
        self.assertEqual(dict_frst['label'], 'a_label')
        # Test second input
        dict_scnd = self.scnd_inp.get_dict()
        self.assertEqual(dict_scnd['type'], 'File')
        self.assertEqual(dict_scnd['inputBinding']['prefix'], '--prefix')
        self.assertEqual(dict_scnd['inputBinding']['position'], 2)


class TestCommandOutputParameter(unittest.TestCase):

    def setUp(self):
        self.outp = cwlgen.CommandOutputParameter('an_out_id', param_type='File')

    def test_init(self):
        self.assertEqual(self.outp.id, 'an_out_id')
        self.assertEqual(self.outp.type, 'File')

    def test_get_dict(self):
        dict_test = self.outp.get_dict()
        self.assertEqual(dict_test['type'], 'File')


class TestCommandLineBinding(unittest.TestCase):

    def setUp(self):
        self.line_binding = cwlgen.CommandLineBinding(load_contents=True, position=1, \
                                                     prefix='--prefix', separate=True, \
                                                     item_separator='-', shell_quote=True,\
                                                     value_from='text.txt')

    def test_init(self):
        self.assertTrue(self.line_binding.loadContents)
        self.assertEqual(self.line_binding.position, 1)
        self.assertEqual(self.line_binding.prefix, '--prefix')
        self.assertTrue(self.line_binding.separate)
        self.assertEqual(self.line_binding.itemSeparator, '-')
        self.assertTrue(self.line_binding.shellQuote)
        self.assertEqual(self.line_binding.valueFrom, 'text.txt')

    def test_get_dict(self):
        dict_test = self.line_binding.get_dict()
        self.assertEqual(dict_test['position'], 1)
        self.assertEqual(dict_test['prefix'], '--prefix')
        self.assertTrue(dict_test['separate'])
        self.assertEqual(dict_test['itemSeparator'], '-')
        self.assertTrue(dict_test['shellQuote'])
        self.assertEqual(dict_test['valueFrom'], 'text.txt')


class TestCommandOutputBinding(unittest.TestCase):

    def setUp(self):
        self.out_binding = cwlgen.CommandOutputBinding(glob='file.txt', load_contents=True,\
                                                      output_eval='eval')

    def test_init(self):
        self.assertEqual(self.out_binding.glob, 'file.txt')
        self.assertTrue(self.out_binding.loadContents)
        self.assertEqual(self.out_binding.outputEval, 'eval')

    def test_get_dict(self):
        dict_test = self.out_binding.get_dict()
        self.assertEqual(dict_test['glob'], 'file.txt')
        self.assertTrue(dict_test['loadContents'])
        self.assertEqual(dict_test['outputEval'], 'eval')


class TestRequirement(unittest.TestCase):

    def setUp(self):
        self.requirement = cwlgen.Requirement('a_class')

    def test_init(self):
        self.assertEqual(self.requirement.req_class, 'a_class')


class TestInlineJavascriptReq(unittest.TestCase):

    def setUp(self):
        self.js_req = cwlgen.InlineJavascriptReq(expression_lib='expression')

    def test_init(self):
        self.assertEqual(self.js_req.req_class, 'InlineJavascriptRequirement')
        self.assertEqual(self.js_req.expressionLib, 'expression')

class TestDockerRequirement(unittest.TestCase):

    def setUp(self):
        self.dock_req = cwlgen.DockerRequirement(docker_pull='pull', docker_load='load',\
                                                docker_file='file', docker_import='import',\
                                                docker_image_id='id', docker_output_dir='dir')

    def test_init(self):
        self.assertEqual(self.dock_req.dockerPull, 'pull')
        self.assertEqual(self.dock_req.dockerLoad, 'load')
        self.assertEqual(self.dock_req.dockerFile, 'file')
        self.assertEqual(self.dock_req.dockerImport, 'import')
        self.assertEqual(self.dock_req.dockerImageId, 'id')
        self.assertEqual(self.dock_req.dockerOutputDir, 'dir')


###########  Main  ###########

if __name__ == "__main__":
    unittest.main()



from setuptools import setup

exec(open('cwlgen/version.py').read())

setup(name="cwlgen",
        version=__version__,
        description='Generation of CWL programmatically. Avaible types: CommandLineTool and DockerRequirement',
        author='Kenzo-Hugo Hillion and Herve Menager',
        author_email='kehillio@pasteur.fr and hmenager@pasteur.fr',
        license='MIT',
        keywords = ['cwl'],
        install_requires=['six', 'ruamel.yaml==0.13.13'],
        packages=["cwlgen"],
        classifiers=[
            'Development Status :: 4 - Beta',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'Environment :: Console',
            ],
        )
